// CustomerFiles.java

/* Author Michael Claudius, Roskilde Computer Science College
 * Version 2.0. 2011.02.07
 * Copyright 2003 by Michael Claudius
 * All rights reserved
 */
package Model;

import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

public class CustomerFiles {

    private Customer aCustomer;
    private CustomerRegister customerReg;
    private String fileName;
    private String inputFileName, vipFileName, otherFileName;

   public CustomerFiles(CustomerRegister customerReg, String fileName) {
        this.customerReg = customerReg;
        this.fileName = fileName;
    }

    public CustomerFiles(String inputFileName, String vipFileName, String otherFileName) {
        this.inputFileName = inputFileName;
        this.vipFileName = vipFileName;
        this.otherFileName = otherFileName;
    }

   public CustomerFiles() {
        customerReg = new CustomerRegister();

    }

    public void writeFile() {

        try {
            FileOutputStream fos = new FileOutputStream(fileName);
            ObjectOutputStream oos = new ObjectOutputStream(fos);

            int number = customerReg.numberOfCustomers();
            oos.writeInt(number);
            for (int index = 0; index < number; index++) {
                aCustomer = customerReg.getCustomer(index);
                oos.writeObject(aCustomer);
            } //for
            oos.close();
        }//try
        catch (IOException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
    }// writeFile

    public void readFile() {

        try {
            FileInputStream fis = new FileInputStream(fileName);
            ObjectInputStream ois = new ObjectInputStream(fis);
            int number = ois.readInt();
            for (int i = 0; i < number; i++) {
                aCustomer = (Customer) ois.readObject();
                customerReg.addCustomer(aCustomer);
            }
            ois.close();
        }//try
        catch (ClassNotFoundException ex) {
            Logger.getLogger(CustomerFiles.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
        }
    }

    
     public void readFile0() {
        ObjectInputStream ois = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(fileName);
            ois = new ObjectInputStream(fis);
            int number = ois.readInt();
            for (int i = 0; i < number; i++) {
                aCustomer = (Customer) ois.readObject();
                customerReg.addCustomer(aCustomer);
            }
            ois.close();
        }//try
        catch (ClassNotFoundException ex) {
            Logger.getLogger(CustomerFiles.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
        } finally {
            try {
                ois.close();
            } catch (IOException ex) {
                Logger.getLogger(CustomerFiles.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public void readFile1() {
        ObjectInputStream ois = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(fileName);
            ois = new ObjectInputStream(fis);
            int number = ois.readInt();
            while (true) {
                aCustomer = (Customer) ois.readObject();
                customerReg.addCustomer(aCustomer);
            }
        }//try
        catch (ClassNotFoundException ex) {
            Logger.getLogger(CustomerFiles.class.getName()).log(Level.SEVERE, null, ex);
        } catch (EOFException eof) {
            System.out.println("EOF read");

        } catch (IOException ex) {
        } finally {
            try {
                ois.close();
            } catch (IOException ex) {
                Logger.getLogger(CustomerFiles.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

// Assignment  Split a file into two files
    /** Reads file into colletion and split this into two collections
     * then writes each collection to a file
     * @param inputFileName input file with customer objects
     * @param vipFileName output file with vip objects
     * @param inputFileName output file with other customer objects
     * @return void
     */
    public void splitFile(String inputFilename, String vipFileName, String otherFileName) {

        CustomerRegister othersReg = new CustomerRegister();
        CustomerRegister vipsReg = new CustomerRegister();

        this.fileName = inputFilename;
        System.out.println(fileName);

        readFile();

        int n = customerReg.numberOfCustomers();
        for (int i = 0; i < n; i++) {
            aCustomer = customerReg.getCustomer(i);
            if ((aCustomer.getCustomerType()).equals("VIP")) {
                vipsReg.addCustomer(aCustomer);
            } else {
                othersReg.addCustomer(aCustomer);
            }
        }

        //Best solution is to utilise the local register and  filename
        customerReg = vipsReg;
        fileName = vipFileName;
        writeFile();

        customerReg = othersReg;
        fileName = otherFileName;
        writeFile();

    }// splitFile

    /** Read file into colletion, traverses the collection and
     * writes objects directly to two files
     */
    public void splitFileClassic(String inutFilename, String vipFileName, String otherFileName) {

        try {
            FileInputStream fis = new FileInputStream(inputFileName);
            ObjectInputStream ois = new ObjectInputStream(fis);

            FileOutputStream fos1 = new FileOutputStream(vipFileName);
            ObjectOutputStream vipsStream = new ObjectOutputStream(fos1);

            FileOutputStream fos2 = new FileOutputStream(otherFileName);
            ObjectOutputStream othersStream = new ObjectOutputStream(fos2);

            //Reads customer from file and add to vips-file or others-file
            int number = ois.readInt();
            for (int i = 0; i < number; i++) {
                aCustomer = (Customer) ois.readObject();
                if ((aCustomer.getCustomerType()).equals("VIP")) {
                    vipsStream.writeObject(aCustomer);
                } else {
                    othersStream.writeObject(aCustomer);
                }
            }

            ois.close();
            vipsStream.close();
            othersStream.close();

        }//try
        catch (IOException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }

    }

    /** Reads file into colletion, traverses the collection and  
     * writes objects to two collections, then writes the collections files.
     * Reuses he readFile and writeFile.
     */
    public void splitFileGeneral() {

        CustomerRegister othersReg = new CustomerRegister();
        CustomerRegister vipsReg = new CustomerRegister();
        CustomerRegister allCustomerReg = new CustomerRegister();

        CustomerFiles cFile = new CustomerFiles(allCustomerReg, inputFileName);
        cFile.readFile();

        int n = allCustomerReg.numberOfCustomers();
        for (int i = 0; i < n; i++) {
            aCustomer = allCustomerReg.getCustomer(i);
            if ((aCustomer.getCustomerType()).equals("VIP")) {
                vipsReg.addCustomer(aCustomer);
            } else {
                othersReg.addCustomer(aCustomer);
            }
        }

        //Best solution is to utilise CustomerFiles
        CustomerFiles cFile1 = new CustomerFiles(vipsReg, vipFileName);
        cFile1.writeFile();

        cFile1 = new CustomerFiles(othersReg, otherFileName);
        cFile1.writeFile();

        //Alternatively


    }// Easy

    public void concatenateFiles(String outputFileName, String vipFileName, String othersFileName) {

        fileName = vipFileName;
        readFile();

        fileName = othersFileName;
        readFile();

        fileName = outputFileName;
        writeFile();
    }

    public void concatenateFilesGeneral(String outputFileName, String vipFileName, String othersFileName) {

        CustomerRegister allCustomerReg = new CustomerRegister();

        CustomerFiles cFile = new CustomerFiles(allCustomerReg, vipFileName);
        cFile.readFile();

        cFile = new CustomerFiles(allCustomerReg, othersFileName);
        cFile.readFile();

        cFile = new CustomerFiles(allCustomerReg, outputFileName);
        cFile.writeFile();
    }

    public void readFile(CustomerRegister customerReg, String fileName) {
        try {
            FileInputStream fis = new FileInputStream(fileName);
            ObjectInputStream ois = new ObjectInputStream(fis);
            customerReg = (CustomerRegister) ois.readObject();
            ois.close();
        }//try
        catch (IOException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
    }

    public Object collectionReadFile(String fileName) {
        try {
            FileInputStream fis = new FileInputStream(fileName);
            ObjectInputStream ois = new ObjectInputStream(fis);
            Object o = ois.readObject();
            ois.close();
            return o;
        } catch (IOException ex) {
            Logger.getLogger(CustomerFiles.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(CustomerFiles.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public void customerCollectionReadFile(CustomerRegister customerReg, String fileName) {
        try {
            FileInputStream fis = new FileInputStream(fileName);
            ObjectInputStream ois = new ObjectInputStream(fis);
            customerReg = (CustomerRegister) ois.readObject();
            ois.close();
        }//try
        catch (IOException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
    }

    public void collectionWriteFile(Object o, String fileName) {
        try {
            FileOutputStream fos = new FileOutputStream(fileName);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(o);
            oos.close();
        }//try
        catch (IOException ex) {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
    }// writeFile
}
